//Program for wh1602a
#include <HDL51001_ccf.h>
#include <uart.h>
#include <wdt.h>

#define RS 2 //PC2
#define E 3 //PC3
#define TIME 200 //pause tick for 80 MHz

void pause(unsigned int a)
{
  unsigned int i;
  for(i=a;i>0;i--);
}

void lcd_com (unsigned char lcd)//Write command to LCD
{
  int j;
  unsigned char temp;
  temp = (lcd&~(1<<RS))|(1<<E); //RS=0 - it's command
  GPIOC->OUT = temp; //Write higth tetrade, RS, E to portA
  for(j=0;j<200;j++); //Settle pause
  GPIOC->OUT = temp&~(1<<E); //Write higth tetrade to LCD
  temp = ((lcd*16)&~(1<<RS))|(1<<E); //RS=0 - it's command
  GPIOC->OUT = temp; //Write low tetrade, RS, E to portA
  for(j=0;j<200;j++) //Settle pause
  GPIOC->OUT = temp&~(1<<E); //Write low tetrade to LCD
  pause(10*TIME); //Pause for executing command
}

void lcd_dat (unsigned char lcd)//Write data to LCD
{
  int j;
  unsigned char temp;
  temp = (lcd|(1<<RS))|(1<<E); //RS=1 - it's data 
  GPIOC->OUT = temp; //Write higth tetrade, RS, E to portA
  for(j=0;j<200;j++); //Settle pause
  GPIOC->OUT = temp&~(1<<E); //Write higth tetrade to LCD
  temp = ((lcd*16)|(1<<RS))|(1<<E); //RS=1 - it's data
  GPIOC->OUT = temp; //Write low tetrade, RS, E to portA
  for(j=0;j<200;j++); //Settle pause
  GPIOC->OUT = temp&~(1<<E); //Write low tetrade to LCD
  pause(TIME); //Pause for loading data
}

void lcd_init (void)    //Init LCD
{
  lcd_com(0x2c); //4-wire data bus, 1-row, 5x8 symbol size
  pause(100*TIME);
  lcd_com(0x0c); //Show image, cursor show
  pause(1000*TIME);
  lcd_com(0x01); //Clear screen and set cursor on 0x00 position
  pause(100*TIME);
}

//display data static
void lcdDat(const char *str)
{
 int i;
 lcd_com(0x01); //Clear screen and set cursor on 0x00 position
 pause(100*TIME);
 for (i = 0; str[i]; i++) lcd_dat(str[i]);
}

//define string length
int strlen(const char *str)
{
 int i, len;
 for (i = 0, len = 0; str[i]; i++) len++;
 return i;
}

//display data dynamic
void lcdDat2(const char *str)
{
 #define SCREENLEN 16
 #define TIMESCREEN 900000
 int i, j, k, len, p;
 char c[SCREENLEN + 1];
 len = strlen(str);
 
 if (len == 0) return;

 j = SCREENLEN - 1;
 while (j > 0)
 {
  for (k = 0; k < j; k++) c[k] = ' ';
  p = SCREENLEN - j;
  if (p > len) p = len;
  for (i = 0; i < p; ++i, ++k) c[k] = str[i];
  c[k] = '\0';
  lcdDat(c);
  pause(TIMESCREEN);
  --j;
 }

 j = 0;
 p = SCREENLEN < len - j ? SCREENLEN : len - j;
 while (p >= 0) // p == 0: clear screen
 {
  for (i = j, k = 0; k < p; ++i, ++k) c[k] = str[i];
  c[k] = '\0';
  lcdDat(c);
  pause(TIMESCREEN);
  ++j;
  p = SCREENLEN < len - j ? SCREENLEN : len - j;
 }


}

char res[1024];
//Buffer for cyrillic symbols
char buf_cyr[] = {0x41, 0xA0-0x100, 0x42, 0xA1-0x100, 0xE0-0x100, 0x45, 0xA3-0x100, 0xA4-0x100, 0xA5-0x100, 0xA6-0x100, 0x4B, 0xA7-0x100, 0x4D, 0x48, 0x4F, 0xA8-0x100, 0x50,
0x43, 0x54, 0xA9-0x100, 0xAA-0x100, 0x58, 0xE1-0x100, 0xAB-0x100, 0xAC-0x100, 0xE2-0x100, 0xAD-0x100, -82, 0x62, 0xAF-0x100, 0xB0-0x100, 0xB1-0x100, 0x61, 0xB2-0x100, 0xB3-0x100, 0xB4-0x100, 0xE3-0x100, 0x65, 0xB6-0x100,
0xB7-0x100, 0xB8-0x100, 0xB9-0x100, 0xBA-0x100, 0xBB-0x100, 0xBC-0x100, 0xBD-0x100, 0x6F, -66, 0x70, 0x63, 0xBF-0x100, 0x79, 0xE4-0x100, 0x78, 0xE5-0x100, 0xC0-0x100, 0xC1-0x100, 0xE6-0x100, 0xC2-0x100, 0xC3-0x100, 0xC4-0x100, 0xC5-0x100,
0xC6-0x100, 0xC7-0x100};

//convert cyrillic symbols to wh1602a symbols
const char* convertRus(const char *str)
{
 int len = strlen(str);
 int i;
 for (i = 0; i < len; i++)
 {
  if (str[i] >= '' && str[i] <= '') res[i] = buf_cyr[64 + str[i]];
  else if (str[i] == '') res[i] = 0xA2-0x100;
       else if (str[i] == '') res[i] = 0xB5-0x100;
            else res[i] = str[i];
 }
 res[i] = '\0';
 
 
 return res;
}


void main()
{

  GPIOB->DIR = ((uint32_t)0x60000000);
  GPIOB->OUT = ((uint32_t)0x60000000); //enable led 2,3
  WDT->KEY = ((uint32_t)0x00003333); //disable wdt
  GPIOC->DIR = ((uint32_t)0x000000FC); //config portC pins to out
  pause(100000);
  lcd_init();
  
 while (1)
 {
 
   lcdDat2(convertRus(""));
   lcdDat2(convertRus("!!!"));

   lcdDat2(convertRus("!    ."));
   lcdDat2(convertRus("  80 ,   4- ."));
   lcdDat2(convertRus("  .!!!"));
//  \0  - symbol of the end string (only for english)
//  lcdDat2("Hello, it's Multiclet. Press any key!!!\0");
	
 }  
 
}


